--[[
	SomeTweaks
	Copyright © 2013, 2015 LudwigV

	This file is part of SomeTweaks.

	SomeTweaks is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	SomeTweaks is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with SomeTweaks.  If not, see <http://www.gnu.org/licenses/>.
]]

local db
local currency = {ids = {241, 390, 61, 515, 398, 384, 81, 615, 393, 392, 361, 402, 395, 754, 416, 677, 614, 400, 394, 397, 676, 391, 401, 385, 396, 399,
		-- MoP
		789, 697, 738, 752, 777, 776,
		-- WoD
		823, 944, 980, 824, 1101, 1129, 994, }}

local function Currency_Init()
	for i = 1, #currency.ids do
		local name = GetCurrencyInfo(currency.ids[i])
		currency[name] = currency.ids[i]
	end
end

local function MoveTalentSpecTabs()
	PlayerSpecTab1Background:SetTexCoord(0, 1, 1, 1, 0, 0, 1, 0)
	PlayerSpecTab2Background:SetTexCoord(0, 1, 1, 1, 0, 0, 1, 0)
	
	hooksecurefunc("PlayerTalentFrame_Update", function()
		PlayerSpecTab1:SetPoint("TOPLEFT", PlayerTalentFrame, "TOPRIGHT", -108, -469)
		PlayerSpecTab2:SetPoint("TOPLEFT", PlayerSpecTab1, "TOPRIGHT", 22, 0)
		PlayerSpecTab1Background:SetPoint("TOPLEFT", -21, 3)
		PlayerSpecTab2Background:SetPoint("TOPLEFT", -21, 3)
	end)
end

local function AddCloseButton()
	local CloseButton = CreateFrame("Button", nil, PlayerTalentFrame, "UIPanelCloseButton")
	CloseButton:SetPoint("BOTTOMRIGHT", -1, -1)
	PlayerTalentFrameActivateButton:SetPoint("RIGHT", CloseButton, "LEFT", 5, 0)
end

local function Backpack_OnClick(self, button, down)
	if not IsModifiedClick() then
		ToggleAllBags()
		BackpackButton_UpdateChecked(self)
	else
		if IsModifiedClick("OPENALLBAGS") then
			BackpackButton_OnClick(self, button)
		end
	end
end

local function Bag_OnClick(self, button, down)
	if not IsModifiedClick() then
		if GetInventoryItemTexture("player", self:GetID()) then
			ToggleAllBags()
		end
		BagSlotButton_UpdateChecked(self)
	else
		if IsModifiedClick("OPENALLBAGS") then
			BagSlotButton_OnClick(self, button)
		end
	end
end

local function ModifyBagButtonOnClickHandler()
	MainMenuBarBackpackButton:SetScript("OnClick", Backpack_OnClick)
	CharacterBag0Slot:SetScript("OnClick", Bag_OnClick)
	CharacterBag1Slot:SetScript("OnClick", Bag_OnClick)
	CharacterBag2Slot:SetScript("OnClick", Bag_OnClick)
	CharacterBag3Slot:SetScript("OnClick", Bag_OnClick)
end

local f = CreateFrame("Frame")
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_LOGOUT")
local function eventHandler(self, event, ...)
	if event == "ADDON_LOADED" then
		if ... == "Blizzard_TalentUI" then
			if db.options[1] then
				MoveTalentSpecTabs()
				AddCloseButton()
			end
		elseif ... == "SomeTweaks" then
			SomeTweaksDB = SomeTweaksDB or {options = {1, 1, 1, 1}, currency = {}}
			db = SomeTweaksDB
			
			SomeTweaksOptions_Init()
			
			if db.options[2] then
				ModifyBagButtonOnClickHandler()
			end
			if db.options[4] then
				StaticPopupDialogs["DELETE_GOOD_ITEM"] = StaticPopupDialogs["DELETE_ITEM"]
			end
		end
		
	elseif event == "PLAYER_LOGIN" then
		if db.options[3] then
			TokenFramePopupGameTooltipCheckBox:Show()
			TokenFramePopup:SetHeight(120)
			Currency_Init()
			GameTooltipAddCurrency()
		end
		
	elseif event == "PLAYER_LOGOUT" then
		db.options = SomeTweaksDB.options
		SomeTweaksDB = db
		
	end
end
f:SetScript("OnEvent", eventHandler)

local cf = CreateFrame("Frame", nil, GameTooltip)
cf:SetAllPoints(GameTooltip)

local function ClearCurrencyFrame()
	if cf.r then
		for r = 1, cf.r do
			for t = 1, 3 do
				_G["GameTooltipTokenFrame"..r.."Token"..t]:Hide()
			end
		end
	end
end

local function CurrencyFrame_Update()
	local name, count, icon
	local r, t = 1, 1
	ClearCurrencyFrame()
	cf.f = cf.f or {}
	cf.r = 0
	local l = GameTooltip:NumLines()
	local d, w = 10, nil
	w = floor((cf:GetWidth() - 2*d)/3 + .5)
	for k, v in pairs(db.currency) do
		if t == 1 then
			if not cf.f[r] then
				cf.f[r] = CreateFrame("Frame", "GameTooltipTokenFrame"..r, cf, "GameTooltipTokenFrameTemplate")
			end
			GameTooltip:AddLine(" ")
			cf.f[r]:SetPoint("LEFT", "GameTooltipTextLeft"..l + r, "LEFT", 0, -1)
		end
		name, count, icon = GetCurrencyInfo(k)
		_G["GameTooltipTokenFrame"..r.."Token"..t].count:SetText(count)
		_G["GameTooltipTokenFrame"..r.."Token"..t].icon:SetTexture(icon)
		_G["GameTooltipTokenFrame"..r.."Token"..t]:SetPoint("RIGHT", cf.f[r], "LEFT", w*t, 0)
		_G["GameTooltipTokenFrame"..r.."Token"..t]:Show()
		t = t + 1
		cf.r = r
		if t == 4 then
			t = 1
			r = r + 1
		end
	end
	cf:Show()
	GameTooltip:Show()
end

function GameTooltipAddCurrency()
	PlayerFrame:HookScript("OnEnter", function()
		CurrencyFrame_Update()
	end)
	
	if IsAddOnLoaded("XPerl_Player") or IsAddOnLoaded("ZPerl_Player") then
		XPerl_PlayernameFrame:HookScript("OnEnter", function()
			CurrencyFrame_Update()
		end)
		XPerl_Player:HookScript("OnEnter", function()
			CurrencyFrame_Update()
		end)
	end
	
	if IsAddOnLoaded("ShadowedUnitFrames") then
		SUFUnitplayer:HookScript("OnEnter", function()
			CurrencyFrame_Update()
		end)
	end
	
	GameTooltip:HookScript("OnShow", function()
		cf:Hide()
	end)
	
	hooksecurefunc("TokenButton_OnClick", function(self)
		if not self.isHeader then
			local name = GetCurrencyListInfo(self.index)
			local c = currency[name]
			TokenFramePopupGameTooltipCheckBox:SetChecked(db.currency[c])
		end
	end)
end

local function SortCurrency()
	local s = {}
	for n in pairs(db.currency) do table.insert(s, n) end
	sort(s)
	db.currency = {}
	for _, v in pairs(s) do db.currency[v] = 1 end
end

function SetCurrencyGameTooltip(id, set)
	local name = GetCurrencyListInfo(id)
	local c = currency[name]
	db.currency[c] = set
	SortCurrency()
	CurrencyFrame_Update()
end
